<?php

namespace Laravelista\Localized;

use Illuminate\Routing\Route;
use Illuminate\Routing\Router;
use Illuminate\Support\ServiceProvider as LaravelServiceProvider;
use Laravelista\Localized\Middleware\SetLocale;

class ServiceProvider extends LaravelServiceProvider
{
    /**
     * Bootstrap any package services.
     */
    public function boot(Router $router, Route $route): void
    {
        $this->publishes([
            __DIR__ . '/../config/localized.php' => $this->app->configPath('localized.php'),
        ], 'config');

        $route->macro('locale', function (string $locale, ?string $identifier = null) {
            $actions = [
                'locale' => $locale,
            ];

            if (!is_null($identifier)) {
                $actions['identifier'] = $identifier;
            }

            /**
             * @var Route $this
             */
            return $this
                ->setAction(array_merge($this->action, $actions))
                ->middleware("set-locale:{$locale}");
        });

        $route->macro('identifier', function (string $identifier) {
            /**
             * @var Route $this
             */
            return $this
                ->setAction(array_merge($this->action, [
                    'identifier' => $identifier
                ]));
        });

        $router->aliasMiddleware('set-locale', SetLocale::class);

        $router->macro('locale', function (string $locale) {
            /**
             * @var Router $this
             */
            return (new RouteRegistrar($this))
                ->locale($locale)
                ->middleware("set-locale:{$locale}");
        });
    }

    /**
     * Register any application services.
     */
    public function register(): void
    {
        $this->mergeConfigFrom(
            __DIR__ . '/../config/localized.php',
            'localized'
        );
    }
}
